==> energy_base-1.7.3-py3-none-any.whl: energy_base/references_api/utils.py <==
import importlib
from re import search, sub

from django.conf import settings
from django.db.models import Model
from rest_framework import filters
from drf_spectacular.utils import extend_schema, extend_schema_serializer
from rest_framework import viewsets, serializers

from ..api.permissions import IsSuperUser
from ..api.views import TranslatedListView

serializer_classes = {}


def find_model(model_name: str):
    for model_class in get_model_classes():
        if get_model_class_name(model_class).lower() == model_name.lower():
            return model_class


def get_api_meta_property(model: Model, property_name: str):
    if hasattr(model, 'ApiMeta'):
        if hasattr(model.ApiMeta, property_name):
            return getattr(model.ApiMeta, property_name)


def get_model_class_name(model: Model):
    return model._meta.object_name


def get_model_fields_list(model: Model):
    return model._meta.get_fields()


def get_view_set_name(model: Model):
    return get_model_class_name(model) + 'ViewSet'


def get_basename(model: Model):
    return 'admin-' + get_plural_form(model)


def camel_to_snake(name: str):
    return ''.join(map(lambda c: '-' + c.lower() if c.isupper() else c, name)).removeprefix('-')


def get_url_prefix(model: Model):
    return 'admin/' + get_plural_form(model)


def get_plural_form(word: str):
    if search(r'[sxz]$', word) or search(r'[^aeioudgkprt]h$', word):
        return sub('$', 'es', word)
    elif search(r'[aeiou]y$', word):
        return sub(r'y$', 'ies', word)
    else:
        return word + 's'


def get_serializer_name(model: Model):
    return get_model_class_name(model) + 'Serializer'


def get_model_classes():
    model_classes = []
    for model_import_path in settings.REFERENCE_API_MODELS:
        module_name, class_name = model_import_path.rsplit('.', 1)
        module = importlib.import_module(module_name)
        model_classes.append(getattr(module, class_name))

    return model_classes


def create_view_set(model: Model):
    @extend_schema(tags=[get_url_prefix(model)])
    class ViewSet(viewsets.ModelViewSet, TranslatedListView):
        http_method_names = get_api_meta_property(model, 'http_method_names') or ['get', 'patch']
        manager = model.objects
        serializer_class = create_serializer(model)
        permission_classes = [IsSuperUser]
        filter_backends = [filters.SearchFilter]
        search_fields = get_model_fields_list(model)

    return ViewSet


def create_serializer(model: Model, depth=0):
    if get_model_class_name(model) in serializer_classes:
        return serializer_classes[get_model_class_name(model)]

    fields_list = get_model_fields_list(model)

    @extend_schema_serializer(component_name='Admin' + get_serializer_name(model))
    class Serializer(serializers.ModelSerializer):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            nonlocal model, fields_list
            for field in fields_list:
                if field.__class__.__name__.endswith('ForeignKey'):
                    model_name = field.name
                    _model = find_model(model_name)
                    if _model is not None and depth == 0:
                        self.fields[model_name] = create_serializer(_model, depth + 1)()

        class Meta:
            fields = '__all__'
            read_only_fields = list(
                {
                    'created_at',
                    'updated_at',
                    'deleted_at',
                    'created_by',
                    'updated_by',
                    'deleted_by',
                    'code'
                } & set([field.name for field in fields_list])
            )

    Serializer.Meta.model = model
    serializer_classes[get_model_class_name(model)] = Serializer
    return Serializer

==> energy_base-1.7.4-py3-none-any.whl: energy_base/references_api/utils.py <==
import importlib
from re import search, sub

from django.conf import settings
from django.db.models import Model, CharField
from rest_framework import filters
from drf_spectacular.utils import extend_schema, extend_schema_serializer
from rest_framework import viewsets, serializers

from ..api.permissions import IsSuperUser
from ..api.views import TranslatedListView

serializer_classes = {}


def find_model(model_name: str):
    for model_class in get_model_classes():
        if get_model_class_name(model_class).lower() == model_name.lower():
            return model_class


def get_api_meta_property(model: Model, property_name: str):
    if hasattr(model, 'ApiMeta'):
        if hasattr(model.ApiMeta, property_name):
            return getattr(model.ApiMeta, property_name)


def get_model_class_name(model: Model):
    return model._meta.object_name


def get_model_fields_list(model: Model):
    return model._meta.get_fields()


def get_view_set_name(model: Model):
    return get_model_class_name(model) + 'ViewSet'


def get_basename(model: Model):
    return 'admin-' + get_plural_form(get_model_class_name(model).lower())


def camel_to_snake(name: str):
    return ''.join(map(lambda c: '-' + c.lower() if c.isupper() else c, name)).removeprefix('-')


def get_url_prefix(model: Model):
    return 'admin/' + get_plural_form(get_model_class_name(model).lower())


def get_plural_form(word: str):
    if search(r'[sxz]$', word) or search(r'[^aeioudgkprt]h$', word):
        return sub('$', 'es', word)
    elif search(r'[aeiou]y$', word):
        return sub(r'y$', 'ies', word)
    else:
        return word + 's'


def get_serializer_name(model: Model):
    return get_model_class_name(model) + 'Serializer'


def get_model_classes():
    model_classes = []
    for model_import_path in settings.REFERENCE_API_MODELS:
        module_name, class_name = model_import_path.rsplit('.', 1)
        module = importlib.import_module(module_name)
        model_classes.append(getattr(module, class_name))

    return model_classes


def create_view_set(model: Model):
    fields_list = get_model_fields_list(model)

    @extend_schema(tags=[get_url_prefix(model)])
    class ViewSet(viewsets.ModelViewSet, TranslatedListView):
        http_method_names = get_api_meta_property(model, 'http_method_names') or ['get', 'patch']
        manager = model.objects
        serializer_class = create_serializer(model)
        permission_classes = []
        filter_backends = [filters.SearchFilter]
        search_fields = [field.name for field in fields_list if isinstance(field, CharField)]

    return ViewSet


def create_serializer(model: Model, depth=0):
    if get_model_class_name(model) in serializer_classes:
        return serializer_classes[get_model_class_name(model)]

    fields_list = get_model_fields_list(model)

    @extend_schema_serializer(component_name='Admin' + get_serializer_name(model))
    class Serializer(serializers.ModelSerializer):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            nonlocal model, fields_list
            for field in fields_list:
                if field.__class__.__name__.endswith('ForeignKey'):
                    model_name = field.name
                    _model = find_model(model_name)
                    if _model is not None and depth == 0:
                        self.fields[model_name] = create_serializer(_model, depth + 1)()

        class Meta:
            fields = '__all__'
            read_only_fields = list(
                {
                    'created_at',
                    'updated_at',
                    'deleted_at',
                    'created_by',
                    'updated_by',
                    'deleted_by',
                    'code'
                } & set([field.name for field in fields_list])
            )

    Serializer.Meta.model = model
    serializer_classes[get_model_class_name(model)] = Serializer
    return Serializer
